<?php
/**
 * @file lib.class.php
 * @author lacovnk
 * @brief class library
 */

/**
 * @brief Page class
 */
class Page
{
	var $title;	/**< title **/
	var $include_header;	/**< header contents **/
	var $include_footer;	/**< footer contents **/
	var $contents;	/**< contents **/
	var $editable;	/**< editable **/

	/// @brief constructor
	function Page($title='TITLE')
	{
		$this->setTitle($title);
		$this->editable = (true == isset($_SESSION['logged']));
	}

	/// @brief set header file
	function setHeaderFile($filename='header.php'){$this->include_header=getParsedContentsAtFile($filename);}
	/// @brief set footer file
	function setFooterFile($filename='footer.php'){$this->include_footer=getParsedContentsAtFile($filename);}
	/// @brief set title
	function setTitle($title){$this->title=$title;}

	/// @brief add string to contents
	function addContents($contents){$this->contents=$this->contents.$contents;}
	/// @brief add file to contents
	function addFile($filename){$this->addContents(getParsedContentsAtFile($filename));}

	/// @brief print header
	function printHeader(){print $this->include_header;}
	/// @brief print contents
	function printContents(){print $this->contents;}
	/// @brief print footer
	function printFooter(){print $this->include_footer;}
	/// @brief print page
	function printPage(){$this->printHeader();print '<h1>'.$this->title.'</h1>';$this->printContents();$this->printFooter();}
	/// @brief make contents - virtual
	function make(){}
}

/**
 * @brief Admin Page Class
 */
class AdminPage extends Page
{
	/// @brief make contents - real
	function make()
	{
		ob_start();
		if(isset($_SESSION['logged'])){print '<a href="action.php?mode=exec_logout">Logout</a>';}
		else
		{
			print <<<BUF
<form method="post" action="action.php?mode=exec_login" enctype="multipart/form-data">
<table>
	<tr><td>password : <input type="password" name="password" id="password"/></td></tr>
	<tr><td><input type="submit" name="submit" value="login" /></td></tr>
</table>
</form>
BUF;
		}
        makeHTML_JS_focus('password');
		$result = ob_get_contents();
		ob_end_clean();
		$this->addContents($result);
	}
}

/**
 * @brief CheckList Page Class
 */
class CheckListPage extends Page
{
	
	/// @brief make contents - real
	function make()
	{
        
		ob_start();
		menu_select_part($part);
		makeHTML_toggleLink('자세한 선택','detail_menu');
		makeHTML_toggleLayer_open('detail_menu');
		menu_select_type($type);
		menu_select_period($begin,$end,'month');
		menu_select_schoolNumber($schoolNumber);
		menu_select_active($active,'1');
		menu_select_regular($regular,NULL);
		makeHTML_toggleLayer_close('detail_menu');

		makeResource_member($part,$schoolNumber,$active,$regular,$member);
		makeResource_time($begin,$end,$type,$time);
		$num_box = 0;
		$seperator_part = new Seperator('part',-1);
		$seperator_partSATB = new Seperator('partSATB',-1);
		$seperator_schoolNumber = new Seperator('schoolNumber',-1);
		if(true == isset($_SESSION['logged']))
		{
			print '<form method="post" action="action.php?mode=exec_check" enctype="multipart/form-data">';
			makeHTML_JS_key();
		}

		print '<table class="border1 inner_border1"><tr><td>Date</td>';
		for($i=0;$i<$time->num_rows;$i++)
		{
			$cur_time = $time->fetch_array();
			$cur_time['shortText'] = split("-",$cur_time['timeDate']);
			//$cur_time['shortText'] = $cur_time['shortText'][1]."<br />".$cur_time['shortText'][2]."<br />".$cur_time['timeLength'];
			$cur_time['shortText'] = $cur_time['shortText'][1]."<br />".$cur_time['shortText'][2]."<br />".$cur_time['timeLength'].'/'.$cur_time['timeOrder'];
			print "<td class='type{$cur_time['timeType']}'>{$cur_time['shortText']}</td>";
		}
		print "</tr>\n";
		$num_colspan = $time->num_rows + 1;	// +1 for memberName
		for($i=0;$i<$member->num_rows;$i++)
		{
			$cur_member = $member->fetch_array();	
			$cur_member_check = new Resource('cur_member_check',query_get_check_memberId_type_period($cur_member['memberId'],$type,$begin,$end));
			$cur_member_partSATB = getSATBNumberFromPartNmber($cur_member['memberPart']);
			$seperator_partSATB->check($cur_member_partSATB,",$num_colspan,'mode=checklist'");
			$seperator_part->check($cur_member['memberPart'],",$cur_member_partSATB,$num_colspan,'mode=checklist'");

			print '<tr onmouseover="style.backgroundColor=\'#CCC\'" onmouseout="style.backgroundColor=\'#FFFFFF\'">';
            print '<td>';
            if($schoolNumber)
            {
                if($seperator_schoolNumber->check($cur_member['memberNumberSchool'])){print substr($cur_member['memberNumberSchool'],2).'&nbsp;';}
                else{print '&nbsp;&nbsp;&nbsp;&nbsp;';}
            };
            print "{$cur_member['memberName']}</td>";
			for($j=0;$j<$time->num_rows;$j++)
			{
				$cur_check = $cur_member_check->fetch_array();
				print '<td>';
				if(true == isset($_SESSION['logged'])){makeHTML_Input_Check($num_box,$cur_check['checkId'],$cur_check['checkResult']);}
				else{print "<img src='./images/{$cur_check['checkResult']}.png' alt='{$cur_check['checkResult']}' />";}
				$num_box++;
				print '</td>';
			}
			print "</tr>\n";
		}
		print '</table>';
		if(true == isset($_SESSION['logged']))
		{
			print <<<BUF
<input type="hidden" name="chk_th" id="chk_th" value="0" />
<input type="hidden" name="chk_ty" id="chk_ty" value="2" />
<input type="hidden" name="num" value="{$num_box}" />
<table class="border1 inner_border1">

	<tr>
		<td>now</td>
		<td colspan="3"><span id="chk_now">1교시 출석</span></td>
	</tr>
	<tr>
		<td>special!</td>
		<td><input type="button" name="full" id="full" value="전부출석(F)" onclick="dothis(this)" /></td>
		<td><input type="button" name="empty" id="empty" value="전부결석(R)" onclick="dothis(this)" /></td>

		<td><input type="submit" value="완료(Enter)" /></td>
	</tr>
	<tr>
		<td>1교시</td>
		<td><input type="button" name="02" id="02" value="출석(Q)" onclick="dothis(this)" /></td><td><input type="button" name="01" id="01" value="공결(W)" onclick="dothis(this)" /></td><td><input type="button" name="00" id="00" value="결석(E)" onclick="dothis(this)" /></td>	</tr>
	<tr>
		<td>2교시</td>
		<td><input type="button" name="12" id="12" value="출석(A)" onclick="dothis(this)" /></td><td><input type="button" name="11" id="11" value="공결(S)" onclick="dothis(this)" /></td><td><input type="button" name="10" id="10" value="결석(D)" onclick="dothis(this)" /></td>	</tr>
	<tr>
		<td>3교시</td>
		<td><input type="button" name="22" id="22" value="출석(A)" onclick="dothis(this)" /></td><td><input type="button" name="21" id="21" value="공결(X)" onclick="dothis(this)" /></td><td><input type="button" name="20" id="20" value="결석(C)" onclick="dothis(this)" /></td>	</tr>
</table>
</form>
BUF;
		}
		$result = ob_get_contents();
		ob_end_clean();
		$this->addContents($result);
	}
}

/**
 * @brief MemberList Page Class
 */
class MemberListPage extends Page
{
	/// @brief make contents - real
	function make()
	{
		ob_start();
		if(isset($_SESSION['logged']))
		{
            menu_select_part($part);
            makeHTML_toggleLink('자세한 선택','detail_menu');
            makeHTML_toggleLayer_open('detail_menu');
            menu_select_schoolNumber($schoolNumber);
            menu_select_active($active,NULL);
            menu_select_regular($regular,NULL);
            makeHTML_toggleLayer_close('detail_menu');

	    // TODO
		$result_time = query_get_time_all();
		$num_result_time = mysql_num_rows($result_time);
		global $data;
		for($i=0;$i<$num_result_time;$i++)
		{
			$cur_time = mysql_fetch_array($result_time);
			$data['SelectTimeId'][$i] = array(0=>$cur_time['timeId'],1=> $cur_time['timeDate']);
		}
            
            makeResource_member($part,$schoolNumber,$active,$regular,$member);
            print $member->getTable('member',$this->editable);
            makeHTML_JS_focus('memberName');
		}
		else {print 'Need Login';}
		$result = ob_get_contents();
		ob_end_clean();
		$this->addContents($result);
	}
}

/**
 * @brief StatisticsList Page Class
 */
class StatisticsListPage extends Page
{
	/// @brief make contents - real
	function make()
	{
		global $config;
		ob_start();
		menu_select_part($part);
		makeHTML_toggleLink('자세한 선택','detail_menu');
		makeHTML_toggleLayer_open('detail_menu');
		menu_select_type($type);
		menu_select_schoolNumber($schoolNumber);
		menu_select_period($begin,$end,'all');
		menu_select_active($active,'1');
		menu_select_regular($regular,NULL);
		makeHTML_toggleLayer_close('detail_menu');

		makeResource_member($part,$schoolNumber,$active,$regular,$member);
		makeResource_time($begin,$end,$type,$time);
		$totalTime = getTotalTimeLength($time);
		$result = array();
		for($i=0;$i<$member->num_rows;$i++)
		{
			$cur_member = $member->fetch_array();	
			$cur_statistics = new Statistics();
			$cur_statistics->setTotalTimeLength($totalTime[0],$totalTime[1]);
			$cur_member_check = new Resource('cur_member_check',query_get_check_memberId_type_period($cur_member['memberId'],$type,$begin,$end));
			for($j=0;$j<$time->num_rows;$j++)
			{
				$cur_check = $cur_member_check->fetch_array();
				$cur_statistics->addCheckResult($cur_check['checkResult'],$cur_check['timeType']);
			}
			$result[$i]['text'] = $cur_statistics->getFullString(true);
			$result[$i]['memberName'] = $cur_member['memberName'];
			$result[$i]['data'] = $cur_statistics->statisticsArray;
			$result[$i]['attendence'] = $cur_statistics->getAttendence(UABSENCE_ADDTOATTENDENCE,false);
			$result[$i]['totalTimeLength'] = $cur_statistics->getTotalTimeLength(UABSENCE_ADDTOATTENDENCE,false);
			$result[$i]['rate'] = $cur_statistics->getRate(UABSENCE_ADDTOATTENDENCE,false);
		}
		my_sort($result,'compare_by_memberName');
		my_sort($result,'compare_by_rate');
		// print
		print '<br />공결은 출석으로 처리 &amp; 보충은 제외<br />';
		print '<table class="border1 inner_border1">';
		print '<tr><td rowspan="2">이름</td><td colspan="3">정기연습</td><td colspan="3">보충연습</td><td rowspan="2"> 출석률</td></tr>';
		print '<tr><td>출석</td><td>공결</td><td>총시간</td>';
		print '<td>출석</td><td>공결</td><td>총시간</td>';
		print '</tr>';
		foreach($result as $cur)
		{
			print '<tr>';
			print "<td>{$cur['memberName']}</td>{$cur['text']}";
			print "<td>{$cur['rate']}% ({$cur['attendence']}/{$cur['totalTimeLength']})</td>";
			print '</tr>';
		}
		print '</table>';

		$result = ob_get_contents();
		ob_end_clean();
		$this->addContents($result);

	}
}

/**
 * @brief TimeList Page Class
 */
class TimeListPage extends Page
{
	/// @brief make contents - real
	function make()
	{
		ob_start();
        menu_select_type($type);
        menu_select_period($begin,$end,'all');
        makeResource_time($begin,$end,$type,$time);

		print $time->getTable('time',$this->editable);
        if(true == $this->editable){makeHTML_JS_focus('timeDate');}
		$result = ob_get_contents();
		ob_end_clean();
		$this->addContents($result);
	}
}

/**
 * @brief Resource Class
 */
class Resource
{
	var $array_fields;
	var $num_rows;
	var $num_fields;
	var $db_resource;
	var $name;

	/// @brief constructor
	function Resource($newName,&$original)
	{
		$this->db_resource = $original;
		$this->num_rows = mysql_num_rows($original);
		$this->num_fields = mysql_num_fields($original);
		$this->name = $newName;
		
		/* set $array_fileds */
		for($i=0;$i<$this->num_fields;$i++)
		{
			$cur_field = mysql_fetch_field($this->db_resource,$i);
			$this->array_fields[sizeof($this->array_fields)] = $cur_field->name;
		}
	}
	function getInfo()
	{
		$this->fetch_array();
		mysql_field_seek($this->db_resource,0);
		return "<pre>num_rows:{$this->num_rows}\rnum_fields:{$this->num_fields}</pre>";
	}
	function printInfo() {print $this->getInfo();}
	function getTable($name,$editable=false,$EditfunctionName=NULL)
	{
		global $data;
		if(0 != mysql_num_rows($this->db_resource)){mysql_data_seek($this->db_resource,0);}
		ob_start();
		print '<table class="border1 inner_border1"><tr>';
		foreach($this->array_fields as $cur){print "<td>{$cur}</td>";}
		if(true == $editable)
		{
			print '<td>Edit</td>';
			print <<<BUF
</tr>
<form method="post" name="action_add" action="action.php?mode=exec_${name}&todo=add" enctype="multipart/form-data">
<tr>
BUF;
			foreach($this->array_fields as $cur)
			{
				print '<td>';
				eval("makeHTML_Input_{$cur}();");
				print '</td>';
			}
			print <<<BUF
	<td>
		<input type="submit" value="Add">
	</td>
</tr>
</form>
BUF;
		}
		else {print '</tr>';}
		for($i=0;$i<$this->num_rows;$i++)
		{
			$cur_row = mysql_fetch_row($this->db_resource);
			print '<tr>';
			for($j=0;$j<$this->num_fields;$j++)
			{
				if($this->array_fields[$j] == 'memberPart'){$temp = $data['NamePart'][$cur_row[$j]];}
				else if($this->array_fields[$j] == 'timeType'){$temp = $data['NameTimeType'][$cur_row[$j]];}
				else{$temp = $cur_row[$j];}
				print "<td>$temp</td>";
			}
			if(true == $editable)
			{
				print <<<BUF
	<td>
		<label class='fakebutton' onclick='window.open("action.php?mode={$this->name}&id={$cur_row[0]}&todo=update","s_sk","width=400,height=500,location=0,menubar=0,resizable=0,scrollbars=0,status=0,toolbar=0");'>Edit</label>
	</td>
BUF;
			}
			print "</tr>\n";
		}
		print '</table>';

		$result = ob_get_contents();

		ob_end_clean();
		if(0 != mysql_num_rows($this->db_resource)){mysql_data_seek($this->db_resource,0);}
		return $result;
	}
	function printTable(){print $this->getTable();}
	function fetch_array(){return mysql_fetch_array($this->db_resource);}
}

/**
 * @brief Seperator Class
 */
class Seperator
{
	var $name;
	var $cur_value;
	/// @brief constructor
	function Seperator($name,$cur_value=-1)
	{
		$this->name = $name;
		$this->cur_value = $cur_value;
	}
	function check($value,$option=NULL)
	{
		if($this->cur_value != $value)
		{
			$this->cur_value = $value;
			return eval("seperator_{$this->name}({$value}{$option});return true;");
		}
	
	}
}

/**
 * @brief Statistics Class
 */
class Statistics
{
	var $statisticsArray = array();
	/// @brief constructor
	function statistics()
	{
		$this->statisticsArray[0]['attendence'] = 0;
		$this->statisticsArray[0]['uabsence'] = 0;
		$this->statisticsArray[0]['totalTimeLength'] = 0;
		$this->statisticsArray[1]['attendence'] = 0;
		$this->statisticsArray[1]['uabsence'] = 0;
		$this->statisticsArray[1]['totalTimeLength'] = 0;
	}
	function setTotalTimeLength($regularLength,$additionalLength)
	{
		$this->statisticsArray[0]['totalTimeLength'] = $regularLength;
		$this->statisticsArray[1]['totalTimeLength'] = $additionalLength;
	}
	function addCheckResult($checkResult,$timeType)
	{
		global $data;
		for($i=0;$i<3;$i++)
		{
			$code = (int)substr($checkResult,$i,1);
			if($code !=0){$this->statisticsArray[$timeType][$data['ConvertCheckResultNumberToStatisticsArraykey'][$code-1]]++;}
		}
	}
	function getAttendence($include_uabsence,$include_additional)
	{
		$result = getAttendenceOptionUabsence($this->statisticsArray[0],$include_uabsence);
		if($include_additional){$result += getAttendenceOptionUabsence($this->statisticsArray[1],$include_uabsence);}
		return $result;
	}
	function getTotalTimeLength($include_uabsence,$include_additional)
	{
		$result = getTotalTimeLengthOptionUabsence($this->statisticsArray[0],$include_uabsence);
		if($include_additional){$result += getTotalTimeLengthOptionUabsence($this->statisticsArray[1],$include_uabsence);}
		return $result;
	}
	function getRate($include_uabsence,$include_additional)
	{
        $mother = $this->getTotalTimeLength($include_uabsence,$include_additional);
        if($mother == 0){return "--.--";}
        $sun = 100 * $this->getAttendence($include_uabsence,$include_additional);
        return number_format($sun / $mother,2);
	}
	function getFullString($include_additional=false)
	{
		ob_start();
		print <<<BUF
<td>{$this->statisticsArray[0]['attendence']}</td>
<td>{$this->statisticsArray[0]['uabsence']}</td>
<td>{$this->statisticsArray[0]['totalTimeLength']}</td>
BUF;
		if(true == $include_additional)
		{
			print <<<BUF
<td>{$this->statisticsArray[1]['attendence']}</td>
<td>{$this->statisticsArray[1]['uabsence']}</td>
<td>{$this->statisticsArray[1]['totalTimeLength']}</td>
BUF;
		}
		$result = ob_get_contents();
		ob_end_clean();
		return $result;
		
	}
}
?>
